module Ctrl
  class Trello
    include HTTParty
    base_uri 'https://api.trello.com/1/'

    def initialize
      @key = ::CONFIG["key"]
      @token = ::CONFIG["token"]
    end

    def get(path, data, options = {})
      options.merge!({query: data, key: @key, token: @token})
      self.class.get(path + "?key=#{@key}&token=#{@token}", options)
    end

    def post(path, data, options = {})
      options.merge!({query: data, key: @key, token: @token})
      self.class.post(path, options)
    end

    def put(path, data, options = {})
      options.merge!({query: data, key: @key, token: @token})
      self.class.put(path, options)
    end

    def delete(path, options = {})
      options.merge!({key: @key, token: @token})
      self.class.delete(path, options)
    end
  end

  module API
    def self.get_boards
      get("/members/me/boards", {filter: "open", lists: "open"})
    end

    def self.get_board(id)
      get("/boards/#{id}", {cards: "open", lists: "open"})
    end

    def self.get_lists_for_board(id)
      get("/boards/#{id}/lists", {filter: "open", cards: "open"})
    end

    def self.get_cards_for_board(id)
      get("/boards/#{id}/cards", {filter: "open"})
    end

    def self.get_list(id)
      get("/lists/#{id}", {cards: "open", board: true})
    end

    def self.get_cards_for_list(id)
      get("/lists/#{id}/cards", {filter: "open", board: true, list: true})
    end

    def self.get_card(id)
      get("/cards/#{id}")
    end

    private

    def self.get(path, data = nil, options = {})
      @@trello ||= Ctrl::Trello.new()
      @@trello.get(path, data, options)
    end

    def self.post(path, data = nil, options = {})
      @@trello ||= Ctrl::Trello.new()
      @@trello.post(path, data, options)
    end

    def self.put(path, data = nil, options = {})
      @@trello ||= Ctrl::Trello.new()
      @@trello.put(path, data, options)
    end

    def self.delete(path, options = {})
      @@trello ||= Ctrl::Trello.new()
      @@trello.delete(path, options)
    end
  end
end
