module Ctrl
  class Card
    attr_accessor :id, :name, :description, :due, :labels, :position, :closed, :url

    def initialize(opts)
      @id = opts["id"]
      @name = opts["name"]
      @description = opts["description"]
      @due = opts["due"]
      @labels = opts["labels"]
      @position = opts["pos"]
      @closed = opts["closed"]
      @url = opts["shortUrl"] || opts["url"]
      @board = Board.new(opts["board"]) if opts["board"]
      @list = List.new(opts["list"]) if opts["list"]
      @board_id = opts["idBoard"]
      @list_id = opts["idList"]
    end

    def save; end

    def self.create(opts); end

    def self.find_by_id(id)
      Card.new(API.get_card(id))
    end

    def self.find_by_name(name, obj)
      if obj.is_a? List
        fuzzy = FuzzyMatch.new(find_all_by_list(obj.id), read: :name)
      elsif obj.is_a? Board
        fuzzy = FuzzyMatch.new(find_all_by_board(obj.id), read: :name)
      else
        fail "Object must be a card or a board"
      end
      fuzzy.find(name)
    end

    def self.find_by_position(pos, list)
      pos = 0 if pos == "first" || pos == "top"
      pos = -1 if pos == "last" || pos == "bottom"

      unless pos.is_a? Integer
        fail "Position must by integer, 'top', 'first', 'bottom', or 'last'"
      end

      find_all_by_list(list.id).sort_by(&:position)[pos]
    end

    def self.find_all_by_list(list_id);
      API.get_cards_for_list(list_id).map { |card| Card.new(card) }
    end

    def self.find_all_by_board(board_id)
      API.get_cards_for_board(board_id).map { |card| Card.new(card) }
    end

    def board
      @board ||= Board.find_by_id(@board_id)
    end

    def list
      @list ||= List.find_by_id(@list_id)
    end

    # TODO
    #def checklists; end
    #def position=(pos); end
    #def move(list, pos=nil); end
    #def print; end
    #def archive; end
    #def delete; end
  end
end
